package	D2P::Generator::PHPClasses;
use base 'D2P::Generator';
use D2P::Parser;

sub	strToPgId
{
	my	$string	= shift;

	return $string if $string =~ /^\".*\"$/;

	return "\"$string\"" if (length $string < 32);

	$string =~ s/\_+/_/g;
	return "\"$string\"" if (length $string < 32);

	my	@list	= split /_/, $string;
	my	$old	= $string;
	my	$lsz	= @list;
	my	$rest	= 32 - @list;
	my	$max;

	$string = "";
	foreach my $it (@list)
	{
		$max = $rest / $lsz;
		$max =~ s/\.(.*)$//;
		$max += 1 if ($1 && $1 > 5 && $max < $rest);
		$string .= '_' if $string;

		if	(length $it > $max)
		{
			$string .= substr $it, 0, $max;
			$rest -= $max;
		}
		else
		{
			$string .= $it;
			$rest -= length $string;
		}

		$lsz--;
	}

	print STDERR "WARNING: name \"$old\" changed to \"$string\"\n";
	return "\"$string\"";
}


###
## Strip double quotes from a Postgres name
#
sub	stripDQuotes
{
	my $n = shift;
	$n =~ s/\"//g;
	return	$n;
}


sub	init
{
	my	$this	= shift;

	$this->SUPER::init();
	$this->{config}->{phpout} = "";
}


sub	commandLine
{
	return	(
		"-p|--php"	=> {
			desc	=> "Output PHP mirror classes to the specified file.",
			uniq	=> 1,
			args	=> ["phpout"]
		}
	);
}


###
## Create complete PHP output
#
sub	generate
{
	my	$this	= shift;
	my	$parser	= shift;
	my	$schema	= $parser->schema();
	my	$phpout = $this->{config}->{phpout};

	return unless $phpout;
	unless	(open(PHPOUT, ">$phpout"))
	{
		print STDERR "WARNING : could not create PHP output file $phpout\n";
		return;
	}
	$this->createPHPHeader($schema, $parser->file());
	$this->createPHPClasses($schema);
	$this->createPHPFooter();
	close PHPOUT;
}


###
## Create PHP header
#
sub	createPHPHeader
{
	my	$this	= shift;
	my	$sch	= shift;
	my	$infile	= shift;
	print PHPOUT "<?php\n\n/*\n";

	my	$title = $sch->header();
	$title =~ s/\n$//;
	print PHPOUT "$title\n\n" if $title;

	print PHPOUT "Input file  : $infile\n";
	print PHPOUT "\n";
	print PHPOUT "Defines PHP objects for the following modules : \n";
	foreach my $mod (@{$sch->modules()->{ordered}})
	{
		$title = $mod->name();
		$title =~ s/\s+/ /g;
		print PHPOUT " - $title\n";
	}

	my	@requiredModules = map { s/\s+/ /g; } keys(%{$sch->extModules()->{byname}});
	if	(@requiredModules)
	{
		print PHPOUT "\n";
		print PHPOUT "Requires the following modules :\n - ";
		print PHPOUT join ("\n - ", @requiredModules);
		print PHPOUT "\n";
	}
	print PHPOUT "*/\n\nrequire_once(\"d2p_manager.inc\");\n";
	print PHPOUT "\n\n";
}


###
## Create PHP classes for each table
#
sub	createPHPClasses
{
	my	$this	= shift;
	my	$sch	= shift;

	foreach my $mod (@{$sch->modules()->{ordered}})
	{
		my	$title = $mod->name();
		$title =~ s/\n/\n\/\/ /g;
		print PHPOUT "\n\n\n// Tables for $title\n\n";
		foreach my $o (@{$mod->tables()->{ordered}})
		{
			my	$x			= 0;
			my	$on			= $o->name();
			my	$tnam			= &strToPgId($on);

			print PHPOUT "\$d2p_cl_$on = \&\$d2pManager->newTable(";
			if	(@{$o->{ancestors}})
			{
				print PHPOUT "\"" . $o->{ancestors}->[0]->{to}->{name} . "\"";
			}
			else
			{
				print PHPOUT "NULL";
			}
			print PHPOUT ", \"$on\", $tnam);\n";
			foreach my $f ($o->getFields())
			{
				my	$fnam = &strToPgId($f->fldName());
				print PHPOUT "\$d2p_cl_$on\->addField($fnam, \"";
				print PHPOUT $f->fldType() . "\", ". ($f->notNull() ? "true" : "false");
				if	($f->{reference})
				{
					my	$table	= strToPgId($f->reference()->table()->name());
					print PHPOUT ", $table";
				}
				print PHPOUT ");\n";
			}
			my	@opk = map { &strToPgId($_->fldName()) } @{$o->pkey()};
			print PHPOUT "\$d2p_cl_$on\->setIdentifier(";
			if	(@opk > 1)
			{
				print PHPOUT "array(" . join(", ", @opk) . ")";
			}
			else
			{
				print PHPOUT $opk[0];
			}
			print PHPOUT ");\n\n";
		}
	}
}


###
## Create PHP footer
#
sub	createPHPFooter
{
	print PHPOUT "\n\n// PHP code generated by dia2postgres.pl\n";
	print PHPOUT "// $VERSION\n"; 
	print PHPOUT "// on " . `date`;
	print PHPOUT "\n?>\n";
}




1;
