package	D2P::Generator::PostgreSQL;
use base 'D2P::Generator';
use D2P::Parser;


sub	strToPgId
{
	my	$string	= shift;

	return $string if $string =~ /^\".*\"$/;

	return "\"$string\"" if (length $string < 32);

	$string =~ s/\_+/_/g;
	return "\"$string\"" if (length $string < 32);

	my	@list	= split /_/, $string;
	my	$old	= $string;
	my	$lsz	= @list;
	my	$rest	= 32 - @list;
	my	$max;

	$string = "";
	foreach my $it (@list)
	{
		$max = $rest / $lsz;
		$max =~ s/\.(.*)$//;
		$max += 1 if ($1 && $1 > 5 && $max < $rest);
		$string .= '_' if $string;

		if	(length $it > $max)
		{
			$string .= substr $it, 0, $max;
			$rest -= $max;
		}
		else
		{
			$string .= $it;
			$rest -= length $string;
		}

		$lsz--;
	}

	print STDERR "WARNING: name \"$old\" changed to \"$string\"\n";
	return "\"$string\"";
}


sub	init
{
	my	$this	= shift;

	$this->SUPER::init();
	$this->{config}->{outfile} = "";
	$this->{config}->{sqlfiles} = [];
}


sub	commandLine
{
	my	$this	= shift;

	return	(
		"-o|--output"	=> {
			desc	=> "Output the PostgreSQL script to the specified file.",
			args	=> ["outfile"],
			uniq	=> 1
		},
		"-s|--sql"	=> {
			desc	=> "Add the contents of the specified file at the end of the script. "
					. "May be repeated for each file to include to the script.",
			argl	=> "sqlfiles",
			args	=> ["sqlfile"],
			uniq	=> 0
		}
	);
}


sub	generate
{
	my	$this	= shift;
	my	$parser	= shift;
	my	$schema = $parser->{schema};

	$this->{outfile} = $this->{config}->{outfile};
	$this->{sqlfiles} = [ map { $_->{sqlfile} } @{$this->{config}->{sqlfiles}} ];

	return unless $this->{outfile};

	## Prepare output
	my	@customCode = ();
	if	(-f $this->{outfile})
	{
		my	($s, @old);

		open(IN, $this->{outfile}) or die "$this->{outfile} : $!\n";
		@old = <IN>;
		close(IN);

		open (SQLOLD, ">$this->{outfile}\~");
		print SQLOLD @old;
		close SQLOLD;

		$s = 0;
		foreach my $l (@old)
		{
			if	(!$s && ($l eq "-- You may insert additionnal SQL code here (DO NOT REMOVE THIS LINE)\n"))
			{
				$s = 1;
			}
			elsif	($s && ($l eq "-- End of additionnal SQL code (DO NOT REMOVE THIS LINE)\n"))
			{
				last;
			}
			elsif	($s)
			{
				push @customCode, $l;
			}
		}
	}
	$this->{customCode} = [@customCode];
	open(SQLOUT, ">$this->{outfile}");

	$this->createSQLHeader($schema, $parser->file());
	$this->createSQLTables($schema);
	$this->copySQLFiles();
	$this->createSQLFooter($schema);

	close SQLOUT;
}


###
## Create SQL header
#
sub	createSQLHeader
{
	my	$this	= shift;
	my	$sch	= shift;
	my	$infile	= shift;

	# Schema header
	my	$title = $sch->header();
	$title =~ s/\n$//;
	my	@lines = split /\n/, $title;
	foreach my $l (@lines)
		{ print SQLOUT "-- $l\n"; }
	print SQLOUT "\n" if @lines;

	# Files
	print SQLOUT "-- Input file  : $infile\n";
	print SQLOUT "-- Output file : " . $this->{outfile} . "\n";
	print SQLOUT "\n";

	# Modules list
	print SQLOUT "-- Defines the following modules : \n";
	foreach my $mod (@{$sch->modules()->{ordered}})
	{
		my	$title = $mod->name();
		$title =~ s/\s+/ /g;

		print SQLOUT "--  $title\n";
	}
	print SQLOUT "\n";

	# Dependencies
	if	(keys %{$sch->extModules()->{byname}})
	{
		print SQLOUT "-- Requires the following modules :\n--  ";
		print SQLOUT join ("\n--  ", keys %{$sch->extModules()->{byname}});
		print SQLOUT "\n\n";
	}
	print SQLOUT "BEGIN TRANSACTION;\n\n\n";
}


###
## Create SQL tables for each included module
#
sub	createSQLTables
{
	my	$this	= shift;
	my	$sch	= shift;

	foreach my $mod (@{$sch->modules()->{ordered}})
	{
		my	$title = $mod->name();
		$title =~ s/\n/\n-- /g;
		print SQLOUT "-- Tables for $title\n\n";

		foreach my $t (@{$mod->tables()->{ordered}})
		{
			my	$x = 0;
			my	@pkey = ();
			my	@references = ();
			my	$tnam = strToPgId($t->name());

			print SQLOUT "\n--  Table " . $t->name() . "\n";
			print SQLOUT "CREATE TABLE $tnam (";

			## Table fields
			foreach my $f (@{$t->fields()})
			{
				my	$fnam = strToPgId($f->fldName());
				$x ? (print SQLOUT ",") : ($x=1);
				print SQLOUT "\n\t$fnam";
				my	$l = 56 - length $fnam;
				if	($l <= 0)
				{
					print SQLOUT " ";
				}
				else
				{
					my	$nt = $l / 8;
					$nt++ if $nt =~ s/\..*$//;
					print SQLOUT "\t" x $nt;
				}
				print SQLOUT $f->fldType();

				# NOT NULL constraint
				print SQLOUT " NOT NULL"
					if	($f->notNull());

				# Default value
				print SQLOUT " DEFAULT " . $f->default()
					if	$f->default() ne "";

				# Prepare reference if required
				if	($f->reference())
				{
					my	$field	= strToPgId($f->reference()->fldName());
					my	$table	= strToPgId($f->reference()->table()->name());
					my	$tmpStr = "'$tnam', '$fnam', '$table', '$field', ";

					$tmpStr .= $f->reference()->deleteCascade() ? "true" : "false";
					$tmpStr .= ", ";
					$tmpStr .= $f->reference()->updateCascade() ? "true" : "false";
					push @references, $tmpStr;
				}
			}

			# Table primary key
			if	(@{$t->pkey()})
			{
				my	@opk = map { strToPgId($_->fldName()) } (@{$t->pkey()});
				print SQLOUT ",\n\tPRIMARY KEY(" . join(",", @opk) . ")";
			}
			print SQLOUT "\n)";

			# Table ancestors
			if	(@{$t->ancestors()})
			{
				my	$y = 0;

				print SQLOUT " INHERITS (";
				foreach my $inhRel (@{$t->ancestors()})
				{
					my	$super = $inhRel->to();
					$y ? (print SQLOUT ", ") : ($y = 1);
					print SQLOUT strToPgId($super->name());
				}
				print SQLOUT ")";
			}
			print SQLOUT ";\n\n";

			# Create indices for this table
			foreach my $i ($t->getIndices())
			{
				print SQLOUT "CREATE " . ($i->isUnique() ? 'UNIQUE ' : '') . "INDEX ";
				print SQLOUT strToPgId($i->{name}) . " ON $tnam";

				#$i->{type} && print SQLOUT " USING " . $i->{type}; FIXME no longer supported

				print SQLOUT " (" . join(',', map { strToPgId($_->fldName()); } $i->getFields()) . ");\n";
			}
			print SQLOUT "\n" if $t->getIndices();

			# Table contents if available
			foreach my $cnt (@{$t->{contents}})
			{
				print SQLOUT "INSERT INTO $tnam(" . join(",", map { strToPgId($_) } keys(%$cnt)) . ")\n";
				print SQLOUT "\tVALUES (" . join(",", values(%$cnt)) . ");\n";
			}
			print SQLOUT "\n" if @{$t->{contents}};

			# D2P SQL library stuff for list and references
			$tnam =~ s/\"//g;
			print SQLOUT "INSERT INTO d2p_lst_list(d2p_name, dia_name) VALUES ('$tnam', '$t->{name}');\n";
			foreach my $ref (@references)
			{
				$ref =~ s/\"//g;
				print SQLOUT "INSERT INTO d2p_ref_list(src_table, src_field, "
					. "dest_table, dest_field, del_cascade, upd_cascade)\n";
				print SQLOUT "\tVALUES ($ref);\n";
			}
			print SQLOUT "\n";
		}
		print SQLOUT "\n";
	}
}


###
## Copy additional SQL files
#
sub	copySQLFiles
{
	my	$this	= shift;
	return unless @{$this->{sqlfiles}};

	print SQLOUT "\n\n-- \n-- ADDITIONAL SQL FILES\n-- \n";
	foreach my $f (@{$this->{sqlfiles}})
	{
		unless	(open(SFIC, $f))
		{
			print STDERR "WARNING : could not open SQL file '$f'\n";
			next;
		}
		print SQLOUT "\n-- \n-- SQL code from $f\n-- \n\n";
		my @cff;
		@cff = <SFIC>;
		close SFIC;
		print SQLOUT @cff;
	}
}


###
## Create SQL footer
#
sub	createSQLFooter
{
	my	$this	= shift;

	print SQLOUT "\n\n-- You may insert additionnal SQL code here (DO NOT REMOVE THIS LINE)\n";
	if	(@{$this->{customCode}})
	{
		print SQLOUT @{$this->{customCode}};
	}
	else
	{
		print SQLOUT "\n" x 4;
	}
	print SQLOUT "-- End of additionnal SQL code (DO NOT REMOVE THIS LINE)\n\n";
	print SQLOUT "SELECT f_d2p_inh_update();\nSELECT f_d2p_ref_update();\n\n";
	print SQLOUT "COMMIT TRANSACTION;\n";
	print SQLOUT "-- SQL code generated by dia2postgres.pl\n";
	print SQLOUT "-- $VERSION\n"; 
	print SQLOUT "-- on " . `date`;
}




1;
